﻿using System;
using System.Collections;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.IO;
using System.Linq;
using System.Windows.Forms;
using System.Xml;

using Lassalle.Flow;

namespace TestProject
{
    /// <summary>
    /// AddFlow 헬퍼
    /// </summary>
    public static class AddFlowHelper
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Class
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 마이 노드 - MyNode

        /// <summary>
        /// 마이 노드
        /// </summary>
        private class MyNode : Node
        {
            //////////////////////////////////////////////////////////////////////////////////////////////////// Field
            ////////////////////////////////////////////////////////////////////////////////////////// Public

            #region Field

            /// <summary>
            /// 핀 리스트
            /// </summary>
            public List<PointF> pinList = new List<PointF>();

            #endregion

            //////////////////////////////////////////////////////////////////////////////////////////////////// Constructor
            ////////////////////////////////////////////////////////////////////////////////////////// Public

            #region 생성자 - MyNode(defaultNode)

            /// <summary>
            /// 생성자
            /// </summary>
            /// <param name="defaultNode">디폴트 노드</param>
            public MyNode(Node defaultNode) : base(0, 0, 0, 0, string.Empty, defaultNode)
            {
            }

            #endregion
        }

        #endregion
        #region 마이 링크 - MyLink

        /// <summary>
        /// 마이 링크
        /// </summary>
        private class MyLink : Link
        {
            //////////////////////////////////////////////////////////////////////////////////////////////////// Field
            ////////////////////////////////////////////////////////////////////////////////////////// Public

            #region Field

            /// <summary>
            /// 시작 노드 인덱스
            /// </summary>
            public int StartNodeIndex = 0;

            /// <summary>
            /// 종료 노드 인덱스
            /// </summary>
            public int EndNodeIndex = 0;

            /// <summary>
            /// 포인트 리스트
            /// </summary>
            public List<PointF> PointList = new List<PointF>();

            #endregion

            //////////////////////////////////////////////////////////////////////////////////////////////////// Constructor
            ////////////////////////////////////////////////////////////////////////////////////////// Public

            #region 생성자 - MyLink(defaultLink)

            /// <summary>
            /// 생성자
            /// </summary>
            /// <param name="defaultLink">디폴트 링크</param>
            public MyLink(Link defaultLink) : base(null, null, null, defaultLink)
            {
            }

            #endregion

            ////////////////////////////////////////////////////////////////////////////////////////// Private

            #region MyLink()

            /// <summary>
            /// 생성자
            /// </summary>
            private MyLink()
            {
            }

            #endregion
        }

        #endregion
        #region 마이 캡션 - MyCaption

        /// <summary>
        /// 마이 캡션
        /// </summary>
        private class MyCaption : Caption
        {
            //////////////////////////////////////////////////////////////////////////////////////////////////// Field
            ////////////////////////////////////////////////////////////////////////////////////////// Public

            #region Field

            /// <summary>
            /// 소유자 인덱스
            /// </summary>
            public int OwnerIndex = 0;

            #endregion

            //////////////////////////////////////////////////////////////////////////////////////////////////// Constructor
            ////////////////////////////////////////////////////////////////////////////////////////// Public

            #region 생성자 - MyCaption(defaultCaption)

            /// <summary>
            /// 생성자
            /// </summary>
            /// <param name="defaultCaption">디폴트 캡션</param>
            public MyCaption(Caption defaultCaption) : base(0, 0, 0, 0, string.Empty, null, defaultCaption)
            {
            }

            #endregion

            ////////////////////////////////////////////////////////////////////////////////////////// Private

            #region 생성자 - MyCaption()

            /// <summary>
            /// 생성자
            /// </summary>
            private MyCaption()
            {
            }

            #endregion
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Enumeration
        ////////////////////////////////////////////////////////////////////////////////////////// Private

        #region 로드 상태 - LoadState

        /// <summary>
        /// 로드 상태
        /// </summary>
        private enum LoadState
        {
            Initial,
            AddFlow,
            NodeModel,
            LinkModel,
            CaptionModel,
            Node,
            Link,
            Caption
        };

        #endregion
        #region 화살표 각도 - ArrowAngle

        /// <summary>
        /// 화살표 각도
        /// </summary>
        private enum ArrowAngle
        {
            /// <summary>
            /// 15도
            /// </summary>
            Degree15,

            /// <summary>
            /// 30도
            /// </summary>
            Degree30,

            /// <summary>
            /// 45도
            /// </summary>
            Degree45,
        }

        #endregion
        #region 화살표 크기 - ArrowSize

        /// <summary>
        /// 화살표 크기
        /// </summary>
        private enum ArrowSize
        {
            /// <summary>
            /// 매우 작은 화살표 크기
            /// </summary>
            VerySmall,

            /// <summary>
            /// 작은 화살표 크기
            /// </summary>
            Small,

            /// <summary>
            /// 중간 화살표 크기
            /// </summary>
            Medium,

            /// <summary>
            /// 큰 화살표 크기
            /// </summary>
            Large,
        };

        #endregion
        #region 점프 - Jump

        /// <summary>
        /// 점프
        /// </summary>
        private enum Jump
        {
            /// <summary>
            /// 점프 표시 안함
            /// </summary>
            None,

            /// <summary>
            /// 링크들 교차시 표시되는 작은 호
            /// </summary>
            Arc,

            /// <summary>
            /// 링크들 교차시 표시되는 선에서 브레이크
            /// </summary>
            Break,
        };

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Static
        //////////////////////////////////////////////////////////////////////////////// Public

        #region 직렬화 하기 - Searialize(xmlTextWriter, addFlow)

        /// <summary>
        /// 직렬화 하기
        /// </summary>
        /// <param name="xmlTextWriter">XmlTextWriter</param>
        /// <param name="addFlow">AddFlow</param>
        /// <remarks>이미지 컬렉션은 저장되지 않는다.</remarks>
        public static void Searialize(XmlTextWriter xmlTextWriter, AddFlow addFlow)
        {
            WriteToXML(xmlTextWriter, addFlow, false);
        }

        #endregion
        #region 직렬화 하기 - Searialize(xmlTextWriter, addFlow, selected)

        /// <summary>
        /// 직렬화 하기
        /// </summary>
        /// <param name="xmlTextWriter">XmlTextWriter</param>
        /// <param name="addFlow">AddFlow</param>
        /// <param name="selected">선택 항목만 저장 여부</param>
        public static void Searialize(XmlTextWriter xmlTextWriter, AddFlow addFlow, bool selected)
        {       
            WriteToXML(xmlTextWriter, addFlow, selected);
        }

        #endregion

        #region 역직렬화 하기 - Deserialize(xmlTextReader, addFlow)

        /// <summary>
        /// 역직렬화 하기
        /// </summary>
        /// <param name="xmlTextReader">XmlTextReader</param>
        /// <param name="addFlow">AddFlow</param>
        /// <remarks>
        /// 이미지는 로드되지 않기 때문에 이미지 컬렉션은 변경되지 않을 것이다.
        /// XML 파일에서 정의된 노드와 링크 속성들을 위한 디폴트 값들은 또한 컨트롤의 디폴트 값들이 될 것이다.
        /// </remarks>
        public static void Deserialize(XmlTextReader xmlTextReader, AddFlow addFlow)
        {
            ReadFromXML(xmlTextReader, addFlow, false, false);
        }

        #endregion
        #region 역직렬화 하기 - Deserialize(xmlTextReader, addFlow, selected)

        /// <summary>
        /// 역직렬화 하기
        /// </summary>
        /// <param name="xmlTextReader">XmlTextReader</param>
        /// <param name="addFlow">AddFlow</param>
        /// <param name="selected">로드 항목 선택 여부</param>
        public static void Deserialize(XmlTextReader xmlTextReader, AddFlow addFlow, bool selected)
        {     
            ReadFromXML(xmlTextReader, addFlow, selected, true);
        }

        #endregion
        #region 역직렬화 하기 - Deserialize(xmlTextReader, addFlow, selected, acceptDefaultValues

        /// <summary>
        /// 역직렬화 하기
        /// </summary>
        /// <param name="xmlTextReader">XmlTextReader</param>
        /// <param name="addFlow">AddFlow</param>
        /// <param name="selected">로드 항목 선택 여부</param>
        /// <param name="acceptDefaultValues">디폴트 값들 수용 여부</param>
        /// <remarks>
        /// 디폴트 값들 수용 여부는 XML 파일에 정의된 노드와 링크 속성들을 위한 디폴트 값들이 컨트롤의 디폴트 값들이 되거나,
        /// 이 디폴트 값들이 로드 작업 중만 사용되는지 여부를 지정한다.
        /// </remarks>
        public static void Deserialize(XmlTextReader xmlTextReader, AddFlow addFlow, bool selected, bool acceptDefaultValues)
        {     
            ReadFromXML(xmlTextReader, addFlow, selected, acceptDefaultValues);
        }

        #endregion 

        //////////////////////////////////////////////////////////////////////////////// Private

        #region 헤더 XML 쓰기 - WriteHeaderToXML(xmlTextWriter, addFlow, selected)

        /// <summary>
        /// 헤더 XML 쓰기
        /// </summary>
        /// <param name="xmlTextWriter">XmlTextWriter</param>
        /// <param name="addFlow">AddFlow</param>
        /// <param name="selected">선택 노드 저장 여부</param>
        private static void WriteHeaderToXML(XmlTextWriter xmlTextWriter, AddFlow addFlow, bool selected)
        {
            int nodeCount    = 0;
            int linkCount    = 0;
            int captionCount = 0;

            if(selected)
            {
                foreach(Item item in addFlow.SelectedItems)
                {
                    if(item is Node)
                    {
                        nodeCount++;
                    }
                    else if(item is Link)
                    {
                        Link link = item as Link;

                        if(link.Org.IsSelected && link.Dst.IsSelected)
                        {
                            linkCount++;
                        }
                    }
                    else if(item is Caption)
                    {
                        captionCount++;
                    }
                }
            }
            else
            {
                Node[]    nodeArray    = addFlow.Items.OfType<Node>().ToArray();
                Link[]    linkArray    = addFlow.Items.OfType<Link>().ToArray();
                Caption[] captionArray = addFlow.Items.OfType<Caption>().ToArray();

                nodeCount    = nodeArray.Count();
                linkCount    = linkArray.Count();
                captionCount = captionArray.Count();
            }

            xmlTextWriter.WriteAttributeString("Nodes"   , XmlConvert.ToString(nodeCount   ));
            xmlTextWriter.WriteAttributeString("Links"   , XmlConvert.ToString(linkCount   ));
            xmlTextWriter.WriteAttributeString("Captions", XmlConvert.ToString(captionCount));
        }

        #endregion
        #region 디폴트 노드 XML 쓰기 - WriteDefaultNodeToXML(xmlTextWriter, defaultNode)

        /// <summary>
        /// 디폴트 노드 XML 쓰기
        /// </summary>
        /// <param name="xmlTextWriter">XmlTextWriter</param>
        /// <param name="defaultNode">디폴트 노드</param>
        private static void WriteDefaultNodeToXML(XmlTextWriter xmlTextWriter, Node defaultNode)
        {
			xmlTextWriter.WriteStartElement("NodeModel");

            if(!defaultNode.Font.Equals(Form.DefaultFont))
            {
                WrtieFontToXML(xmlTextWriter, defaultNode.Font);
            }

            if(!defaultNode.FillColor.Equals(SystemColors.Window))
            {
                xmlTextWriter.WriteElementString("FillColor", defaultNode.FillColor.ToArgb().ToString());
            }

            if(!defaultNode.DrawColor.Equals(SystemColors.WindowText))
            {
                xmlTextWriter.WriteElementString("DrawColor", defaultNode.DrawColor.ToArgb().ToString());
            }

            if(!defaultNode.TextColor.Equals(SystemColors.WindowText))
            {
                xmlTextWriter.WriteElementString("TextColor", defaultNode.TextColor.ToArgb().ToString());
            }

            if(!defaultNode.GradientColor.Equals(SystemColors.Control))
            {
                xmlTextWriter.WriteElementString("GradientColor", defaultNode.GradientColor.ToArgb().ToString());
            }

            if(defaultNode.ShadowStyle != ShadowStyle.None)
            {
                xmlTextWriter.WriteElementString("ShadowStyle", defaultNode.ShadowStyle.ToString());
            }

            if(defaultNode.ShadowColor != SystemColors.GrayText)
            {
                xmlTextWriter.WriteElementString("ShadowColor", defaultNode.ShadowColor.ToArgb().ToString());
            }

            if(defaultNode.ShadowSize.Width != SystemInformation.IconSize.Width / 4 || defaultNode.ShadowSize.Height != SystemInformation.IconSize.Height / 4)
            {
                WriteShadowSizeToXML(xmlTextWriter, defaultNode.ShadowSize);
            }

            if(defaultNode.ShapeFamily != ShapeFamily.Ellipse)
            {
                xmlTextWriter.WriteElementString("ShapeFamily", defaultNode.ShapeFamily.ToString());
            }

            if(defaultNode.TextPosition != TextPosition.CenterMiddle)
            {
                xmlTextWriter.WriteElementString("TextPosition", defaultNode.TextPosition.ToString());
            }

            if(defaultNode.Thickness != 1)
            {
                xmlTextWriter.WriteElementString("Thickness", defaultNode.Thickness.ToString());
            }

            if(defaultNode.DashStyle != DashStyle.Solid)
            {
                xmlTextWriter.WriteElementString("DashStyle", defaultNode.DashStyle.ToString());
            }

            if(defaultNode.BackMode != BackMode.Transparent)
            {
                xmlTextWriter.WriteElementString("BackMode", defaultNode.BackMode.ToString());
            }

            if(defaultNode.GradientMode != LinearGradientMode.BackwardDiagonal)
            {
                xmlTextWriter.WriteElementString("GradientMode", defaultNode.GradientMode.ToString());
            }

            if(defaultNode.ShouldSerializeTextMargin())
            {
                WriteTextMarginToXML(xmlTextWriter, defaultNode.TextMargin);
            }

            if(defaultNode.ShouldSerializeImageMargin())
            {
                WriteImageMarginToXML(xmlTextWriter, defaultNode.ImageMargin);
            }

            if(defaultNode.ImagePosition != ImagePosition.CenterMiddle)
            {
                xmlTextWriter.WriteElementString("ImagePosition", defaultNode.ImagePosition.ToString());
            }

            if(defaultNode.TextPosition != TextPosition.CenterMiddle)
            {
                xmlTextWriter.WriteElementString("TextPosition", defaultNode.TextPosition.ToString());
            }


            if(!defaultNode.IsHitTestVisible)
            {
                xmlTextWriter.WriteElementString("IsHitTestVisible", "False");
            }

            if(!defaultNode.IsSelectable)
            {
                xmlTextWriter.WriteElementString("IsSelectable", "False");
            }

            if(defaultNode.IsOwnerDraw)
            {
                xmlTextWriter.WriteElementString("IsOwnerDraw", "True");
            }

            if(!defaultNode.IsXMoveable)
            {
                xmlTextWriter.WriteElementString("IsXMoveable", "False");
            }

            if(!defaultNode.IsYMoveable)
            {
                xmlTextWriter.WriteElementString("IsYMoveable", "False");
            }

            if(!defaultNode.IsXSizeable)
            {
                xmlTextWriter.WriteElementString("IsXSizeable", "False");
            }

            if(!defaultNode.IsYSizeable)
            {
                xmlTextWriter.WriteElementString("IsYSizeable", "False");
            }

            if(!defaultNode.IsEditable)
            {
                xmlTextWriter.WriteElementString("IsEditable", "False");
            }

            if(!defaultNode.IsInLinkable)
            {
                xmlTextWriter.WriteElementString("IsInLinkable", "False");
            }

            if(!defaultNode.IsOutLinkable)
            {
                xmlTextWriter.WriteElementString("IsOutLinkable", "False");
            }

            if(!defaultNode.IsCaptionsHilighted)
            {
                xmlTextWriter.WriteElementString("IsCaptionsHilighted", "False");
            }


            if(defaultNode.Tooltip != null && defaultNode.Tooltip.Length > 0)
            {
                xmlTextWriter.WriteElementString("Tooltip", defaultNode.Tooltip);
            }

            if(defaultNode.Text != null && defaultNode.Text.Length > 0)
            {
                xmlTextWriter.WriteElementString("Text", defaultNode.Text);
            }
            if(defaultNode.Image != null)
            {
                WriteImageToXML(xmlTextWriter, defaultNode.Image);
            }


            if(defaultNode.ShouldSerializePins())
            {
                WritePinsToXML(xmlTextWriter, defaultNode);
            }

            xmlTextWriter.WriteEndElement();
        }

        #endregion
        #region 디폴트 링크 XML 쓰기 - WriteDefaultLinkToXML(xmlTextWriter, deaultLink)

        /// <summary>
        /// 디폴트 링크 XML 쓰기
        /// </summary>
        /// <param name="xmlTextWriter">XmlTextWriter</param>
        /// <param name="deaultLink">디폴트 링크</param>
        private static void WriteDefaultLinkToXML(XmlTextWriter xmlTextWriter, Link deaultLink)
        {
            xmlTextWriter.WriteStartElement("LinkModel");

            if(!deaultLink.Font.Equals(Form.DefaultFont))
            {
                WrtieFontToXML(xmlTextWriter, deaultLink.Font);
            }

            if(!deaultLink.FillColor.Equals(SystemColors.Window))
            {
                xmlTextWriter.WriteElementString("FillColor", deaultLink.FillColor.ToArgb().ToString());
            }

            if(!deaultLink.DrawColor.Equals(SystemColors.WindowText))
            {
                xmlTextWriter.WriteElementString("DrawColor", deaultLink.DrawColor.ToArgb().ToString());
            }

            if(!deaultLink.TextColor.Equals(SystemColors.WindowText))
            {
                xmlTextWriter.WriteElementString("TextColor", deaultLink.TextColor.ToArgb().ToString());
            }


            if(deaultLink.LineStyle != LineStyle.Polyline)
            {
                xmlTextWriter.WriteElementString("Style", deaultLink.LineStyle.ToString());
            }

            if(deaultLink.RoundedCornerSize != 0f)
            {
                xmlTextWriter.WriteElementString("RoundedCornerSize", deaultLink.RoundedCornerSize.ToString());
            }

            if(deaultLink.JumpSize != 3f)
            {
                xmlTextWriter.WriteElementString("JumpSize", deaultLink.JumpSize.ToString());
            }

            if(deaultLink.BackMode != BackMode.Transparent)
            {
                xmlTextWriter.WriteElementString("BackMode", deaultLink.BackMode.ToString());
            }

            if(deaultLink.Thickness != 1)
            {
                xmlTextWriter.WriteElementString("Thickness", deaultLink.Thickness.ToString());
            }

            if(deaultLink.DashStyle != DashStyle.Solid)
            {
                xmlTextWriter.WriteElementString("DashStyle", deaultLink.DashStyle.ToString());
            }

            if(deaultLink.ShadowStyle != ShadowStyle.None)
            {
                xmlTextWriter.WriteElementString("ShadowStyle", deaultLink.ShadowStyle.ToString());
            }

            if(deaultLink.ShadowColor != SystemColors.GrayText)
            {
                xmlTextWriter.WriteElementString("ShadowColor", deaultLink.ShadowColor.ToArgb().ToString());
            }

            if(deaultLink.ShadowSize.Width != SystemInformation.IconSize.Width / 4 || deaultLink.ShadowSize.Height != SystemInformation.IconSize.Height / 4)
            {
                WriteShadowSizeToXML(xmlTextWriter, deaultLink.ShadowSize);
            }


            if(!deaultLink.IsHitTestVisible)
            {
                xmlTextWriter.WriteElementString("IsHitTestVisible", "False");
            }

            if(!deaultLink.IsSelectable)
            {
                xmlTextWriter.WriteElementString("IsSelectable", "False");
            }

            if(deaultLink.IsOwnerDraw)
            {
                xmlTextWriter.WriteElementString("IsOwnerDraw", "True");
            }

            if(deaultLink.IsOrientedText)
            {
                xmlTextWriter.WriteElementString("IsOrientedText", "True");
            }

            if(deaultLink.IsAdjustOrg)
            {
                xmlTextWriter.WriteElementString("IsAdjustOrg", "True");
            }

            if(deaultLink.IsAdjustDst)
            {
                xmlTextWriter.WriteElementString("IsAdjustDst", "True");
            }

            if(!deaultLink.IsStretchable)
            {
                xmlTextWriter.WriteElementString("IsStretchable", "False");
            }

            if(!deaultLink.IsArrowDst)
            {
                xmlTextWriter.WriteElementString("IsArrowDst", "False");
            }

            if(deaultLink.IsArrowOrg)
            {
                xmlTextWriter.WriteElementString("IsArrowOrg", "True");
            }

            if(deaultLink.IsArrowMid)
            {
                xmlTextWriter.WriteElementString("IsArrowMid", "True");
            }

            if(!deaultLink.IsCaptionsHilighted)
            {
                xmlTextWriter.WriteElementString("IsCaptionsHilighted", "False");
            }


            if(deaultLink.Tooltip != null && deaultLink.Tooltip.Length > 0)
            {
                xmlTextWriter.WriteElementString("Tooltip", deaultLink.Tooltip);
            }

            if(deaultLink.Text != null && deaultLink.Text.Length > 0)
            {
                xmlTextWriter.WriteElementString("Text", deaultLink.Text);
            }

            xmlTextWriter.WriteEndElement();
        }

        #endregion
        #region 디폴트 캡션 XML 쓰기 - WriteDefaultCaptionToXML(xmlTextWriter, defaultCaption)

        /// <summary>
        /// 디폴트 캡션 XML 쓰기
        /// </summary>
        /// <param name="xmlTextWriter">XmlTextWriter</param>
        /// <param name="defaultCaption">디폴트 캡션</param>
        private static void WriteDefaultCaptionToXML(XmlTextWriter xmlTextWriter, Caption defaultCaption)
        {
            xmlTextWriter.WriteStartElement("CaptionModel");

            if(!defaultCaption.Font.Equals(Form.DefaultFont))
            {
                WrtieFontToXML(xmlTextWriter, defaultCaption.Font);
            }

            if(!defaultCaption.FillColor.Equals(SystemColors.Window))
            {
                xmlTextWriter.WriteElementString("FillColor", defaultCaption.FillColor.ToArgb().ToString());
            }

            if(!defaultCaption.DrawColor.Equals(SystemColors.WindowText))
            {
                xmlTextWriter.WriteElementString("DrawColor", defaultCaption.DrawColor.ToArgb().ToString());
            }

            if(!defaultCaption.TextColor.Equals(SystemColors.WindowText))
            {
                xmlTextWriter.WriteElementString("TextColor", defaultCaption.TextColor.ToArgb().ToString());
            }


            if(!defaultCaption.GradientColor.Equals(SystemColors.Control))
            {
                xmlTextWriter.WriteElementString("GradientColor", defaultCaption.GradientColor.ToArgb().ToString());
            }

            if(defaultCaption.ShadowStyle != ShadowStyle.None)
            {
                xmlTextWriter.WriteElementString("ShadowStyle", defaultCaption.ShadowStyle.ToString());
            }

            if(defaultCaption.ShadowColor != SystemColors.GrayText)
            {
                xmlTextWriter.WriteElementString("ShadowColor", defaultCaption.ShadowColor.ToArgb().ToString());
            }

            if(defaultCaption.ShadowSize.Width != SystemInformation.IconSize.Width / 4 || defaultCaption.ShadowSize.Height != SystemInformation.IconSize.Height / 4)
            {
                WriteShadowSizeToXML(xmlTextWriter, defaultCaption.ShadowSize);
            }

            if(defaultCaption.ShapeFamily != ShapeFamily.Ellipse)
            {
                xmlTextWriter.WriteElementString("ShapeFamily", defaultCaption.ShapeFamily.ToString());
            }

            if(defaultCaption.TextPosition != TextPosition.CenterMiddle)
            {
                xmlTextWriter.WriteElementString("TextPosition", defaultCaption.TextPosition.ToString());
            }

            if(defaultCaption.Thickness != 1)
            {
                xmlTextWriter.WriteElementString("Thickness", defaultCaption.Thickness.ToString());
            }

            if(defaultCaption.DashStyle != DashStyle.Solid)
            {
                xmlTextWriter.WriteElementString("DashStyle", defaultCaption.DashStyle.ToString());
            }

            if(defaultCaption.BackMode != BackMode.Transparent)
            {
                xmlTextWriter.WriteElementString("BackMode", defaultCaption.BackMode.ToString());
            }

            if(defaultCaption.GradientMode != LinearGradientMode.BackwardDiagonal)
            {
                xmlTextWriter.WriteElementString("GradientMode", defaultCaption.GradientMode.ToString());
            }

            if(defaultCaption.ShouldSerializeTextMargin())
            {
                WriteTextMarginToXML(xmlTextWriter, defaultCaption.TextMargin);
            }

            if(defaultCaption.ShouldSerializeImageMargin())
            {
                WriteImageMarginToXML(xmlTextWriter, defaultCaption.ImageMargin);
            }

            if(defaultCaption.ImagePosition != ImagePosition.CenterMiddle)
            {
                xmlTextWriter.WriteElementString("ImagePosition", defaultCaption.ImagePosition.ToString());
            }

            if(defaultCaption.TextPosition != TextPosition.CenterMiddle)
            {
                xmlTextWriter.WriteElementString("TextPosition", defaultCaption.TextPosition.ToString());
            }

            if(defaultCaption.Dock != DockStyle.None)
            {
                xmlTextWriter.WriteElementString("Dock", defaultCaption.Dock.ToString());
            }

            if(defaultCaption.AnchorPositionOnLink != 0.5f)
            {
                xmlTextWriter.WriteElementString("AnchorPositionOnLink", XmlConvert.ToString(defaultCaption.AnchorPositionOnLink));
            }


            if(!defaultCaption.IsHitTestVisible)
            {
                xmlTextWriter.WriteElementString("IsHitTestVisible", "False");
            }

            if(!defaultCaption.IsSelectable)
            {
                xmlTextWriter.WriteElementString("IsSelectable", "False");
            }

            if(defaultCaption.IsOwnerDraw)
            {
                xmlTextWriter.WriteElementString("IsOwnerDraw", "True");
            }

            if(!defaultCaption.IsXMoveable)
            {
                xmlTextWriter.WriteElementString("IsXMoveable", "False");
            }

            if(!defaultCaption.IsYMoveable)
            {
                xmlTextWriter.WriteElementString("IsYMoveable", "False");
            }

            if(!defaultCaption.IsXSizeable)
            {
                xmlTextWriter.WriteElementString("IsXSizeable", "False");
            }

            if(!defaultCaption.IsYSizeable)
            {
                xmlTextWriter.WriteElementString("IsYSizeable", "False");
            }

            if(!defaultCaption.IsEditable)
            {
                xmlTextWriter.WriteElementString("IsEditable", "False");
            }


            if(defaultCaption.Tooltip != null && defaultCaption.Tooltip.Length > 0)
            {
                xmlTextWriter.WriteElementString("Tooltip", defaultCaption.Tooltip);
            }

            if(defaultCaption.Text != null && defaultCaption.Text.Length > 0)
            {
                xmlTextWriter.WriteElementString("Text", defaultCaption.Text);
            }

            if(defaultCaption.Image != null)
            {
                WriteImageToXML(xmlTextWriter, defaultCaption.Image);
            }

            xmlTextWriter.WriteEndElement();
        }

        #endregion
        #region 노드 XML 쓰기 - WriteNodeToXML(xmlTextWriter, node, index)

        /// <summary>
        /// 노드 XML 쓰기
        /// </summary>
        /// <param name="xmlTextWriter">XmlTextWriter</param>
        /// <param name="node">노드</param>
        /// <param name="index">인덱스</param>
        private static void WriteNodeToXML(XmlTextWriter xmlTextWriter, Node node, int index)
        {
            Node defaultNode = node.AddFlow.NodeModel;

            xmlTextWriter.WriteStartElement("Node");

            RectangleF nodeRectangle = node.Bounds;

            xmlTextWriter.WriteAttributeString("Index" , XmlConvert.ToString(index               ));
            xmlTextWriter.WriteAttributeString("Left"  , XmlConvert.ToString(nodeRectangle.Left  ));
            xmlTextWriter.WriteAttributeString("Top"   , XmlConvert.ToString(nodeRectangle.Top   ));
            xmlTextWriter.WriteAttributeString("Width" , XmlConvert.ToString(nodeRectangle.Width ));
            xmlTextWriter.WriteAttributeString("Height", XmlConvert.ToString(nodeRectangle.Height));

            if(!node.FillColor.Equals(defaultNode.FillColor))
            {
                xmlTextWriter.WriteElementString("FillColor", node.FillColor.ToArgb().ToString());
            }

            if(!node.DrawColor.Equals(defaultNode.DrawColor))
            {
                xmlTextWriter.WriteElementString("DrawColor", node.DrawColor.ToArgb().ToString());
            }

            if(!node.TextColor.Equals(defaultNode.TextColor))
            {
                xmlTextWriter.WriteElementString("TextColor", node.TextColor.ToArgb().ToString());
            }

            if(!node.GradientColor.Equals(defaultNode.GradientColor))
            {
                xmlTextWriter.WriteElementString("GradientColor", node.GradientColor.ToArgb().ToString());
            }

            if(node.BackMode != defaultNode.BackMode)
            {
                xmlTextWriter.WriteElementString("BackMode", node.BackMode.ToString());
            }

            if(node.ShadowStyle != defaultNode.ShadowStyle)
            {
                xmlTextWriter.WriteElementString("ShadowStyle", node.ShadowStyle.ToString());
            }

            if(node.ShadowColor != defaultNode.ShadowColor)
            {
                xmlTextWriter.WriteElementString("ShadowColor", node.ShadowColor.ToArgb().ToString());
            }

            if(node.ShadowSize.Width != defaultNode.ShadowSize.Width || node.ShadowSize.Height != defaultNode.ShadowSize.Height)
            {
                WriteShadowSizeToXML(xmlTextWriter, node.ShadowSize);
            }

            if(node.ShapeFamily != defaultNode.ShapeFamily)
            {
                xmlTextWriter.WriteElementString("ShapeFamily", node.ShapeFamily.ToString());
            }

            if(node.TextPosition != defaultNode.TextPosition)
            {
                xmlTextWriter.WriteElementString("TextPosition", node.TextPosition.ToString());
            }

            if(node.ImagePosition != defaultNode.ImagePosition)
            {
                xmlTextWriter.WriteElementString("ImagePosition", node.ImagePosition.ToString());
            }

            if(node.GradientMode != defaultNode.GradientMode)
            {
                xmlTextWriter.WriteElementString("GradientMode", node.GradientMode.ToString());
            }

            if(node.DashStyle != defaultNode.DashStyle)
            {
                xmlTextWriter.WriteElementString("DashStyle", node.DashStyle.ToString());
            }

            if(node.Thickness != defaultNode.Thickness)
            {
                xmlTextWriter.WriteElementString("Thickness", node.Thickness.ToString());
            }

            if(node.ShouldSerializeTextMargin())
            {
                WriteTextMarginToXML(xmlTextWriter, node.TextMargin);
            }

            if(defaultNode.ShouldSerializeImageMargin())
            {
                WriteImageMarginToXML(xmlTextWriter, node.ImageMargin);
            }


            if(node.IsHitTestVisible != defaultNode.IsHitTestVisible)
            {
                xmlTextWriter.WriteElementString("IsHitTestVisible", node.IsHitTestVisible ? "True" : "False");
            }

            if(node.IsSelectable != defaultNode.IsSelectable)
            {
                xmlTextWriter.WriteElementString("IsSelectable", node.IsSelectable ? "True" : "False");
            }

            if(node.IsOwnerDraw != defaultNode.IsOwnerDraw)
            {
                xmlTextWriter.WriteElementString("IsOwnerDraw", node.IsOwnerDraw ? "True" : "False");
            }

            if(node.IsXMoveable != defaultNode.IsXMoveable)
            {
                xmlTextWriter.WriteElementString("IsXMoveable", node.IsXMoveable ? "True" : "False");
            }

            if(node.IsYMoveable != defaultNode.IsYMoveable)
            {
                xmlTextWriter.WriteElementString("IsYMoveable", node.IsYMoveable ? "True" : "False");
            }

            if(node.IsXSizeable != defaultNode.IsXSizeable)
            {
                xmlTextWriter.WriteElementString("IsXSizeable", node.IsXSizeable ? "True" : "False");
            }

            if(node.IsYSizeable != defaultNode.IsYSizeable)
            {
                xmlTextWriter.WriteElementString("IsYSizeable", node.IsYSizeable ? "True" : "False");
            }

            if(node.IsEditable != defaultNode.IsEditable)
            {
                xmlTextWriter.WriteElementString("IsEditable", node.IsEditable ? "True" : "False");
            }

            if(node.IsInLinkable != defaultNode.IsInLinkable)
            {
                xmlTextWriter.WriteElementString("IsInLinkable", node.IsInLinkable ? "True" : "False");
            }

            if(node.IsOutLinkable != defaultNode.IsOutLinkable)
            {
                xmlTextWriter.WriteElementString("IsOutLinkable", node.IsOutLinkable ? "True" : "False");
            }

            if(node.IsCaptionsHilighted != defaultNode.IsCaptionsHilighted)
            {
                xmlTextWriter.WriteElementString("IsCaptionsHilighted", node.IsCaptionsHilighted ? "True" : "False");
            }


            if(node.Text != null && node.Text.Length > 0)
            {
                xmlTextWriter.WriteElementString("Text", node.Text);
            }

            if(node.Tooltip != null && node.Tooltip.Length > 0)
            {
                xmlTextWriter.WriteElementString("Tooltip", node.Tooltip);
            }


            if(!node.Font.Equals(defaultNode.Font))
            {
                WrtieFontToXML(xmlTextWriter, node.Font);
            }


            if(node.Image != null)
            {
                WriteImageToXML(xmlTextWriter, node.Image);
            }


            if(node.ShouldSerializePins())
            {
                WritePinsToXML(xmlTextWriter, node);
            }

            xmlTextWriter.WriteEndElement();
        }

        #endregion
        #region 링크 XML 쓰기 - WriteLinkToXML(xmlTextWriter, link, startNodeIndex, endNodeIndex, index)

        /// <summary>
        /// 링크 XML 쓰기
        /// </summary>
        /// <param name="xmlTextWriter">XmlTextWriter</param>
        /// <param name="link">링크</param>
        /// <param name="startNodeIndex">시작 노드 인덱스</param>
        /// <param name="endNodeIndex">종료 노드 인덱스</param>
        /// <param name="index">인덱스</param>
        private static void WriteLinkToXML(XmlTextWriter xmlTextWriter, Link link, int startNodeIndex, int endNodeIndex, int index)
        {
            Link defaultLink = link.AddFlow.LinkModel;

            xmlTextWriter.WriteStartElement("Link");

            xmlTextWriter.WriteAttributeString("Index", XmlConvert.ToString(index));
            xmlTextWriter.WriteAttributeString("Org"  , startNodeIndex.ToString() );
            xmlTextWriter.WriteAttributeString("Dst"  , endNodeIndex.ToString()   );


            if(!link.FillColor.Equals(defaultLink.FillColor))
            {
                xmlTextWriter.WriteElementString("FillColor", link.FillColor.ToArgb().ToString());
            }

            if(!link.DrawColor.Equals(defaultLink.DrawColor))
            {
                xmlTextWriter.WriteElementString("DrawColor", link.DrawColor.ToArgb().ToString());
            }

            if(!link.TextColor.Equals(defaultLink.TextColor))
            {
                xmlTextWriter.WriteElementString("TextColor", link.TextColor.ToArgb().ToString());
            }

            if(link.LineStyle != defaultLink.LineStyle)
            {
                xmlTextWriter.WriteElementString("LineStyle", link.LineStyle.ToString());
            }

            if(link.RoundedCornerSize != defaultLink.RoundedCornerSize)
            {
                xmlTextWriter.WriteElementString("RoundedCornerSize", link.RoundedCornerSize.ToString());
            }

            if(link.JumpSize != defaultLink.JumpSize)
            {
                xmlTextWriter.WriteElementString("JumpSize", link.JumpSize.ToString());
            }

            if(link.BackMode != defaultLink.BackMode)
            {
                xmlTextWriter.WriteElementString("BackMode", link.BackMode.ToString());
            }

            if(link.DashStyle != defaultLink.DashStyle)
            {
                xmlTextWriter.WriteElementString("DashStyle", link.DashStyle.ToString());
            }

            if(link.Thickness != defaultLink.Thickness)
            {
                xmlTextWriter.WriteElementString("Thickness", link.Thickness.ToString());
            }

            if(link.ShadowStyle != defaultLink.ShadowStyle)
            {
                xmlTextWriter.WriteElementString("ShadowStyle", link.ShadowStyle.ToString());
            }

            if(link.ShadowColor != defaultLink.ShadowColor)
            {
                xmlTextWriter.WriteElementString("ShadowColor", link.ShadowColor.ToArgb().ToString());
            }

            if(link.ShadowSize.Width != defaultLink.ShadowSize.Width || link.ShadowSize.Height != defaultLink.ShadowSize.Height)
            {
                WriteShadowSizeToXML(xmlTextWriter, link.ShadowSize);
            }


            if(link.IsHitTestVisible != defaultLink.IsHitTestVisible)
            {
                xmlTextWriter.WriteElementString("IsHitTestVisible", link.IsHitTestVisible ? "True" : "False");
            }

            if(link.IsSelectable != defaultLink.IsSelectable)
            {
                xmlTextWriter.WriteElementString("IsSelectable", link.IsSelectable ? "True" : "False");
            }

            if(link.IsOwnerDraw != defaultLink.IsOwnerDraw)
            {
                xmlTextWriter.WriteElementString("IsOwnerDraw", link.IsOwnerDraw ? "True" : "False");
            }

            if(link.IsOrientedText != defaultLink.IsOrientedText)
            {
                xmlTextWriter.WriteElementString("IsOrientedText", link.IsOrientedText ? "True" : "False");
            }

            if(link.IsAdjustOrg != defaultLink.IsAdjustOrg)
            {
                xmlTextWriter.WriteElementString("IsAdjustOrg", link.IsAdjustOrg ? "True" : "False");
            }

            if(link.IsAdjustDst != defaultLink.IsAdjustDst)
            {
                xmlTextWriter.WriteElementString("IsAdjustDst", link.IsAdjustDst ? "True" : "False");
            }

            if(link.IsStretchable != defaultLink.IsStretchable)
            {
                xmlTextWriter.WriteElementString("IsStretchable", link.IsStretchable ? "True" : "False");
            }

            if(link.IsArrowDst != defaultLink.IsArrowDst)
            {
                xmlTextWriter.WriteElementString("IsArrowDst", link.IsArrowDst ? "True" : "False");
            }

            if(link.IsArrowOrg != defaultLink.IsArrowOrg)
            {
                xmlTextWriter.WriteElementString("IsArrowOrg", link.IsArrowOrg ? "True" : "False");
            }

            if(link.IsArrowMid != defaultLink.IsArrowMid)
            {
                xmlTextWriter.WriteElementString("IsArrowMid", link.IsArrowMid ? "True" : "False");
            }

            if(link.IsCaptionsHilighted != defaultLink.IsCaptionsHilighted)
            {
                xmlTextWriter.WriteElementString("IsCaptionsHilighted", link.IsCaptionsHilighted ? "True" : "False");
            }


            if(link.Text != null && link.Text.Length > 0)
            {
                xmlTextWriter.WriteElementString("Text", link.Text);
            }

            if(link.Tooltip != null && link.Tooltip.Length > 0)
            {
                xmlTextWriter.WriteElementString("Tooltip", link.Tooltip);
            }


            if(!link.Font.Equals(defaultLink.Font))
            {
                WrtieFontToXML(xmlTextWriter, link.Font);
            }


            if(link.CountPoints() > 2 || link.IsAdjustOrg || link.IsAdjustDst)
            {
                WritePointsToXML(xmlTextWriter, link);
            }

            xmlTextWriter.WriteEndElement();
        }

        #endregion
        #region 캡션 XML 쓰기 - WriteCaptionToXML(xmlTextWriter, caption, ownerIndex, index)

        /// <summary>
        /// 캡션 XML 쓰기
        /// </summary>
        /// <param name="xmlTextWriter">XmlTextWriter</param>
        /// <param name="caption">캡션</param>
        /// <param name="ownerIndex">소유자 인덱스</param>
        /// <param name="index">인덱스</param>
        private static void WriteCaptionToXML(XmlTextWriter xmlTextWriter, Caption caption, int ownerIndex, int index)
        {
            Caption defaultCaption = caption.AddFlow.CaptionModel;

            xmlTextWriter.WriteStartElement("Caption");

            RectangleF captionRectangle = caption.Bounds;


            xmlTextWriter.WriteAttributeString("Index" , XmlConvert.ToString(index)                  );
            xmlTextWriter.WriteAttributeString("Owner" , ownerIndex.ToString()                       );
            xmlTextWriter.WriteAttributeString("Left"  , XmlConvert.ToString(captionRectangle.Left)  );
            xmlTextWriter.WriteAttributeString("Top"   , XmlConvert.ToString(captionRectangle.Top)   );
            xmlTextWriter.WriteAttributeString("Width" , XmlConvert.ToString(captionRectangle.Width) );
            xmlTextWriter.WriteAttributeString("Height", XmlConvert.ToString(captionRectangle.Height));


            if(!caption.FillColor.Equals(defaultCaption.FillColor))
            {
                xmlTextWriter.WriteElementString("FillColor", caption.FillColor.ToArgb().ToString());
            }

            if(!caption.DrawColor.Equals(defaultCaption.DrawColor))
            {
                xmlTextWriter.WriteElementString("DrawColor", caption.DrawColor.ToArgb().ToString());
            }

            if(!caption.TextColor.Equals(defaultCaption.TextColor))
            {
                xmlTextWriter.WriteElementString("TextColor", caption.TextColor.ToArgb().ToString());
            }

            if(!caption.GradientColor.Equals(defaultCaption.GradientColor))
            {
                xmlTextWriter.WriteElementString("GradientColor", caption.GradientColor.ToArgb().ToString());
            }

            if(caption.BackMode != defaultCaption.BackMode)
            {
                xmlTextWriter.WriteElementString("BackMode", caption.BackMode.ToString());
            }

            if(caption.ShadowStyle != defaultCaption.ShadowStyle)
            {
                xmlTextWriter.WriteElementString("ShadowStyle", caption.ShadowStyle.ToString());
            }

            if(caption.ShadowColor != defaultCaption.ShadowColor)
            {
                xmlTextWriter.WriteElementString("ShadowColor", caption.ShadowColor.ToArgb().ToString());
            }

            if(caption.ShadowSize.Width != defaultCaption.ShadowSize.Width || caption.ShadowSize.Height != defaultCaption.ShadowSize.Height)
            {
                WriteShadowSizeToXML(xmlTextWriter, caption.ShadowSize);
            }

            if(caption.ShapeFamily != defaultCaption.ShapeFamily)
            {
                xmlTextWriter.WriteElementString("ShapeFamily", caption.ShapeFamily.ToString());
            }

            if(caption.TextPosition != defaultCaption.TextPosition)
            {
                xmlTextWriter.WriteElementString("TextPosition", caption.TextPosition.ToString());
            }

            if(caption.ImagePosition != defaultCaption.ImagePosition)
            {
                xmlTextWriter.WriteElementString("ImagePosition", caption.ImagePosition.ToString());
            }

            if(caption.GradientMode != defaultCaption.GradientMode)
            {
                xmlTextWriter.WriteElementString("GradientMode", caption.GradientMode.ToString());
            }

            if(caption.DashStyle != defaultCaption.DashStyle)
            {
                xmlTextWriter.WriteElementString("DashStyle", caption.DashStyle.ToString());
            }

            if(caption.Thickness != defaultCaption.Thickness)
            {
                xmlTextWriter.WriteElementString("Thickness", caption.Thickness.ToString());
            }

            if(caption.ShouldSerializeTextMargin())
            {
                WriteTextMarginToXML(xmlTextWriter, caption.TextMargin);
            }

            if(caption.ShouldSerializeImageMargin())
            {
                WriteImageMarginToXML(xmlTextWriter, caption.ImageMargin);
            }

            if(caption.Dock != defaultCaption.Dock)
            {
                xmlTextWriter.WriteElementString("Dock", caption.Dock.ToString());
            }

            if(caption.AnchorPositionOnLink != defaultCaption.AnchorPositionOnLink)
            {
                xmlTextWriter.WriteElementString("AnchorPositionOnLink", XmlConvert.ToString(caption.AnchorPositionOnLink));
            }


            if(caption.IsHitTestVisible != defaultCaption.IsHitTestVisible)
            {
                xmlTextWriter.WriteElementString("IsHitTestVisible", caption.IsHitTestVisible ? "True" : "False");
            }

            if(caption.IsSelectable != defaultCaption.IsSelectable)
            {
                xmlTextWriter.WriteElementString("IsSelectable", caption.IsSelectable ? "True" : "False");
            }

            if(caption.IsOwnerDraw != defaultCaption.IsOwnerDraw)
            {
                xmlTextWriter.WriteElementString("IsOwnerDraw", caption.IsOwnerDraw ? "True" : "False");
            }

            if(caption.IsXMoveable != defaultCaption.IsXMoveable)
            {
                xmlTextWriter.WriteElementString("IsXMoveable", caption.IsXMoveable ? "True" : "False");
            }

            if(caption.IsYMoveable != defaultCaption.IsYMoveable)
            {
                xmlTextWriter.WriteElementString("IsYMoveable", caption.IsYMoveable ? "True" : "False");
            }

            if(caption.IsXSizeable != defaultCaption.IsXSizeable)
            {
                xmlTextWriter.WriteElementString("IsXSizeable", caption.IsXSizeable ? "True" : "False");
            }

            if(caption.IsYSizeable != defaultCaption.IsYSizeable)
            {
                xmlTextWriter.WriteElementString("IsYSizeable", caption.IsYSizeable ? "True" : "False");
            }

            if(caption.IsEditable != defaultCaption.IsEditable)
            {
                xmlTextWriter.WriteElementString("IsEditable", caption.IsEditable ? "True" : "False");
            }


            if(caption.Text != null && caption.Text.Length > 0)
            {
                xmlTextWriter.WriteElementString("Text", caption.Text);
            }

            if(caption.Tooltip != null && caption.Tooltip.Length > 0)
            {
                xmlTextWriter.WriteElementString("Tooltip", caption.Tooltip);
            }


            if(!caption.Font.Equals(defaultCaption.Font))
            {
                WrtieFontToXML(xmlTextWriter, caption.Font);
            }


            if(caption.Image != null)
            {
                WriteImageToXML(xmlTextWriter, caption.Image);
            }

            xmlTextWriter.WriteEndElement();
        }

        #endregion
        #region 포인트 XML 쓰기 - WritePointsToXML(xmlTextWriter, link)

        /// <summary>
        /// 포인트 XML 쓰기
        /// </summary>
        /// <param name="xmlTextWriter">XmlTextWriter</param>
        /// <param name="link">링크</param>
        private static void WritePointsToXML(XmlTextWriter xmlTextWriter, Link link)
        {
            if(link.IsAdjustOrg)
            {
                xmlTextWriter.WriteStartElement("Point");                

                PointF point = link.Points[0];

                xmlTextWriter.WriteAttributeString("X", XmlConvert.ToString(point.X));
                xmlTextWriter.WriteAttributeString("Y", XmlConvert.ToString(point.Y));

                xmlTextWriter.WriteEndElement();
            }

            for(int k = 1; k < link.Points.Count-1; k++)
            {
                xmlTextWriter.WriteStartElement("Point"); 

                PointF point = link.Points[k];

                xmlTextWriter.WriteAttributeString("X", XmlConvert.ToString(point.X));
                xmlTextWriter.WriteAttributeString("Y", XmlConvert.ToString(point.Y));

                xmlTextWriter.WriteEndElement();
            }

            if(link.IsAdjustDst)
            {
                xmlTextWriter.WriteStartElement("Point"); 

                PointF point = link.Points[link.Points.Count-1];

                xmlTextWriter.WriteAttributeString("X", XmlConvert.ToString(point.X));
                xmlTextWriter.WriteAttributeString("Y", XmlConvert.ToString(point.Y));

                xmlTextWriter.WriteEndElement();
            }
        }

        #endregion
        #region 핀 XML 쓰기 - WritePinsToXML(xmlTextWriter, node)

        /// <summary>
        /// 핀 XML 쓰기
        /// </summary>
        /// <param name="xmlTextWriter">XmlTextWriter</param>
        /// <param name="node">노드</param>
        static void WritePinsToXML(XmlTextWriter xmlTextWriter, Node node)
        {
            for(int k = 0; k < node.Pins.Count; k++)
            {
                xmlTextWriter.WriteStartElement("Pin");

                PointF point = node.Pins[k];

                xmlTextWriter.WriteAttributeString("X", XmlConvert.ToString(point.X));
                xmlTextWriter.WriteAttributeString("Y", XmlConvert.ToString(point.Y));

                xmlTextWriter.WriteEndElement();
            }
        }

        #endregion
        #region 텍스트 마진 XML 쓰기 - WriteTextMarginToXML(xmlTextWriter, padding)

        /// <summary>
        /// 텍스트 마진 XML 쓰기
        /// </summary>
        /// <param name="xmlTextWriter">XmlTextWriter</param>
        /// <param name="padding">패딩</param>
        private static void WriteTextMarginToXML(XmlTextWriter xmlTextWriter, Padding padding)
        {
            xmlTextWriter.WriteStartElement("TextMargin");

            xmlTextWriter.WriteAttributeString("Left"  , padding.Left.ToString()  );
            xmlTextWriter.WriteAttributeString("Top"   , padding.Top.ToString()   );
            xmlTextWriter.WriteAttributeString("Right" , padding.Right.ToString() );
            xmlTextWriter.WriteAttributeString("Bottom", padding.Bottom.ToString());

            xmlTextWriter.WriteEndElement();
        }

        #endregion
        #region 이미지 마진 XML 쓰기 - WriteImageMarginToXML(xmlTextWriter, padding)

        /// <summary>
        /// 이미지 마진 XML 쓰기
        /// </summary>
        /// <param name="xmlTextWriter">XmlTextWriter</param>
        /// <param name="padding">패일</param>
        private static void WriteImageMarginToXML(XmlTextWriter xmlTextWriter, Padding padding)
        {
            xmlTextWriter.WriteStartElement("ImageMargin");

            xmlTextWriter.WriteAttributeString("Left"  , padding.Left.ToString()  );
            xmlTextWriter.WriteAttributeString("Top"   , padding.Top.ToString()   );
            xmlTextWriter.WriteAttributeString("Right" , padding.Right.ToString() );
            xmlTextWriter.WriteAttributeString("Bottom", padding.Bottom.ToString());

            xmlTextWriter.WriteEndElement();
        }

        #endregion
        #region 그림자 크기 XML 쓰기 - WriteShadowSizeToXML(xmlTextWriter, size)

        /// <summary>
        /// 그림자 크기 XML 쓰기
        /// </summary>
        /// <param name="xmlTextWriter">XmlTextWriter</param>
        /// <param name="size">크기</param>
        private static void WriteShadowSizeToXML(XmlTextWriter xmlTextWriter, Size size)
        {
            xmlTextWriter.WriteStartElement("ShadowSize");

            xmlTextWriter.WriteAttributeString("Width" , size.Width.ToString() );
            xmlTextWriter.WriteAttributeString("Height", size.Height.ToString());

            xmlTextWriter.WriteEndElement();
        }

        #endregion
        #region 폰트 XML 쓰기 - WrtieFontToXML(xmlTextWriter, font)
        
        /// <summary>
        /// 폰트 XML 쓰기
        /// </summary>
        /// <param name="xmlTextWriter">XmlTextWriter</param>
        /// <param name="font">폰트</param>
        static void WrtieFontToXML(XmlTextWriter xmlTextWriter, Font font)
        {
            xmlTextWriter.WriteStartElement("Font");

            xmlTextWriter.WriteAttributeString("Name"     , font.Name                        );
            xmlTextWriter.WriteAttributeString("Size"     , XmlConvert.ToString(font.Size)   );
            xmlTextWriter.WriteAttributeString("Bold"     , font.Bold ? "True" : "False"     );
            xmlTextWriter.WriteAttributeString("Italic"   , font.Italic ? "True" : "False"   );
            xmlTextWriter.WriteAttributeString("Strikeout", font.Strikeout ? "True" : "False");
            xmlTextWriter.WriteAttributeString("Underline", font.Underline ? "True" : "False");
            xmlTextWriter.WriteEndElement();
        }

        #endregion
        #region 이미지 XML 쓰기 - WriteImageToXML(xmlTextWriter, image)

        /// <summary>
        /// 이미지 XML 쓰기
        /// </summary>
        /// <param name="xmlTextWriter">XmlTextWriter</param>
        /// <param name="image">이미지</param>
        private static void WriteImageToXML(XmlTextWriter xmlTextWriter, Image image)
        {
            TypeConverter typeConverter = TypeDescriptor.GetConverter(typeof(Bitmap));

            string valueString = Convert.ToBase64String((byte[])typeConverter.ConvertTo(image, typeof(byte[])));

            xmlTextWriter.WriteElementString("Image", valueString);
        }

        #endregion

        #region XML 쓰기 - WriteToXML(xmlTextWriter, addFlow, selected)

        /// <summary>
        /// XML 쓰기
        /// </summary>
        /// <param name="xmlTextWriter">XmlTextWriter</param>
        /// <param name="addFlow">AddFlow</param>
        /// <param name="selected">선택 항목만 저장 여부</param>
        private static void WriteToXML(XmlTextWriter xmlTextWriter, AddFlow addFlow, bool selected)
        {      
            xmlTextWriter.Formatting = Formatting.Indented;

            xmlTextWriter.WriteStartDocument();

            xmlTextWriter.WriteComment("AddFlow.net diagram");

            xmlTextWriter.WriteStartElement("AddFlow");
                        
            WriteHeaderToXML(xmlTextWriter, addFlow, selected);

            xmlTextWriter.WriteElementString("Version", addFlow.ProductVersion);

            if(addFlow.ShouldSerializeNodeModel())
            {
                WriteDefaultNodeToXML(xmlTextWriter, addFlow.NodeModel);
            }
            
            if(addFlow.ShouldSerializeLinkModel())
            {
                WriteDefaultLinkToXML(xmlTextWriter, addFlow.LinkModel);
            }

            if(addFlow.ShouldSerializeCaptionModel())
            {
                WriteDefaultCaptionToXML(xmlTextWriter, addFlow.CaptionModel);
            }
            
            IList itemList = selected ? addFlow.SelectedItems : addFlow.Items;

            foreach(Item item in itemList)
            {
                int index = itemList.IndexOf(item);

                if(item is Node)
                {
                    Node node = item as Node;

                    WriteNodeToXML(xmlTextWriter, node, index);
                }
                else if(item is Link)
                {
                    Link link = item as Link;

                    int startIndex = itemList.IndexOf(link.Org);
                    int endIndex   = itemList.IndexOf(link.Dst);

                    if(startIndex >= 0 && endIndex >= 0)
                    {
                        WriteLinkToXML(xmlTextWriter, link, startIndex, endIndex, index);
                    }
                }
                else if(item is Caption)
                {
                    Caption caption = item as Caption;

                    int ownerIndex = itemList.IndexOf(caption.Owner);

                    WriteCaptionToXML(xmlTextWriter, caption, ownerIndex, index);
                }
            }

            xmlTextWriter.WriteEndElement();
            xmlTextWriter.WriteEndDocument();
            xmlTextWriter.Flush();
        }

        #endregion

        #region 화살표 구하기 - GetArrow(xmlTextReader)

        /// <summary>
        /// 화살표 구하기
        /// </summary>
        /// <param name="xmlTextReader">XmlTextReader</param>
        /// <returns>그래픽스 패스</returns>
        private static GraphicsPath GetArrow(XmlTextReader xmlTextReader)
        {
            LinkArrowStyle linkArrowStyle = (LinkArrowStyle)Enum.Parse(typeof(LinkArrowStyle), xmlTextReader.GetAttribute(0), false);
            ArrowSize      arrowSize      = (ArrowSize)Enum.Parse(typeof(ArrowSize), xmlTextReader.GetAttribute(1), false);
            ArrowAngle     arrowAngle     = (ArrowAngle)Enum.Parse(typeof(ArrowAngle), xmlTextReader.GetAttribute(2), false);
            float          unit           = SystemInformation.IconSize.Width / 4;
            float          sizeArrowUnit  = (float)Math.Sqrt(unit * unit + unit * unit);
            float          sizeArrow      = (((int)arrowSize + 1) * sizeArrowUnit) / 2;
            float          height;
            float          weight;

            switch(arrowAngle)
            {
                case ArrowAngle.Degree45 :

                    weight = sizeArrow * (float)Math.Cos(Math.PI / 4);
                    height = sizeArrow * (float)Math.Sin(Math.PI / 4);

                    break;

                case ArrowAngle.Degree30 :

                    weight = sizeArrow * (float)Math.Cos(Math.PI / 6);
                    height = sizeArrow * (float)Math.Sin(Math.PI / 6);

                    break;

                case ArrowAngle.Degree15 :
                default                  :

                    weight = sizeArrow * (float)Math.Cos(Math.PI / 12);
                    height = sizeArrow * (float)Math.Sin(Math.PI / 12);

                    break;
            }

            return PredefinedGeometry.GetLinkArrowStylePath(linkArrowStyle, weight, height);
        }

        #endregion
        #region 텍스트 마진 XML 읽기 - ReadTextMarginFromXML(xmlTextReader)

        /// <summary>
        /// 텍스트 마진 XML 읽기
        /// </summary>
        /// <param name="xmlTextReader">XmlTextReader</param>
        /// <returns>텍스트 마진</returns>
        static Padding ReadTextMarginFromXML(XmlTextReader xmlTextReader)
        {
            int left   = 0;
            int top    = 0;
            int right  = 0;
            int bottom = 0;

            if(xmlTextReader.GetAttribute("Left") != null)
            {
                left = XmlConvert.ToInt32(xmlTextReader.GetAttribute("Left"));
            }

            if(xmlTextReader.GetAttribute("Top") != null)
            {
                top = XmlConvert.ToInt32(xmlTextReader.GetAttribute("Top"));
            }

            if(xmlTextReader.GetAttribute("Right") != null)
            {
                right = XmlConvert.ToInt32(xmlTextReader.GetAttribute("Right"));
            }

            if(xmlTextReader.GetAttribute("Bottom") != null)
            {
                bottom = XmlConvert.ToInt32(xmlTextReader.GetAttribute("Bottom"));
            }

            return new Padding(left, top, right, bottom);
        }

        #endregion
        #region 이미지 마진 XML 읽기 - ReadImageMarginFromXML(xmlTextReader)

        /// <summary>
        /// 이미지 마진 XML 읽기
        /// </summary>
        /// <param name="xmlTextReader">XmlTextReader</param>
        /// <returns>이미지 마진</returns>
        private static Padding ReadImageMarginFromXML(XmlTextReader xmlTextReader)
        {
            return new Padding
            (
                XmlConvert.ToInt32(xmlTextReader.GetAttribute("Left"  )),
                XmlConvert.ToInt32(xmlTextReader.GetAttribute("Top"   )),
                XmlConvert.ToInt32(xmlTextReader.GetAttribute("Right" )),
                XmlConvert.ToInt32(xmlTextReader.GetAttribute("Bottom"))
            );
        }

        #endregion
        #region 그림자 크기 XML 읽기 - ReadShadowSizeFromXML(xmlTextReader)

        /// <summary>
        /// 그림자 크기 XML 읽기
        /// </summary>
        /// <param name="xmlTextReader">XmlTextReader</param>
        /// <returns>그림자 크기</returns>
        private static Size ReadShadowSizeFromXML(XmlTextReader xmlTextReader)
        {
            return new Size
            (
                XmlConvert.ToInt32(xmlTextReader.GetAttribute("Width" )),
                XmlConvert.ToInt32(xmlTextReader.GetAttribute("Height"))
            );
        }

        #endregion 
        #region 폰트 XML 읽기 - ReadFontFromXML(xmlTextReader)

        /// <summary>
        /// 폰트 XML 읽기
        /// </summary>
        /// <param name="xmlTextReader">XmlTextReader</param>
        /// <returns>폰트 XML</returns>
        private static Font ReadFontFromXML(XmlTextReader xmlTextReader)
        {         
            string    familyName = xmlTextReader.GetAttribute(0);
            float     size       = XmlConvert.ToSingle(xmlTextReader.GetAttribute(1));
            FontStyle fontStyle  = FontStyle.Regular;
            bool      bold       = (xmlTextReader.GetAttribute(2) == "True");

            if(bold)
            {
                fontStyle = fontStyle | FontStyle.Bold;
            }

            bool italic = (xmlTextReader.GetAttribute(3) == "True");

            if(italic)
            {
                fontStyle = fontStyle | FontStyle.Italic;
            }

            bool strikeout = (xmlTextReader.GetAttribute(4) == "True");

            if(strikeout)
            {
                fontStyle = fontStyle | FontStyle.Strikeout;
            }

            bool underline = (xmlTextReader.GetAttribute(5) == "True");

            if(underline)
            {
                fontStyle = fontStyle | FontStyle.Underline;
            }

            return new Font(familyName, size, fontStyle);
        }

        #endregion 
        #region 이미지 XML 읽기 - ReadImageFromXML(xmlTextReader)
        
        /// <summary>
        /// 이미지 XML 읽기
        /// </summary>
        /// <param name="xmlTextReader">XmlTextReader</param>
        /// <returns>이미지</returns>
        private static Image ReadImageFromXML(XmlTextReader xmlTextReader)
        {
            MemoryStream memoryStream = new MemoryStream();

            byte[] buffetArray = new byte[1000];   

            int readCount = xmlTextReader.ReadBase64(buffetArray, 0, 1000);

            while(0 != readCount)
            {   
                memoryStream.Write(buffetArray, 0, readCount);

                readCount = xmlTextReader.ReadBase64(buffetArray, 0, 1000); 
            } 

            return Image.FromStream(memoryStream);
        }

        #endregion 

        #region XML 읽기 - ReadFromXML(xmlTextReader, addFlow, selected, acceptDefaultValues)

        /// <summary>
        /// XML 읽기
        /// </summary>
        /// <param name="xmlTextReader">XmlTextReader</param>
        /// <param name="addFlow">AddFlow</param>
        /// <param name="selected">로드 항목 선택 여부</param>
        /// <param name="acceptDefaultValues">디폴트 값들 수용 여부</param>
        private static void ReadFromXML(XmlTextReader xmlTextReader, AddFlow addFlow, bool selected, bool acceptDefaultValues)
        {           
            Node    defaultNode    = new Node(0, 0, 0, 0, null, addFlow);
            Link    defaultLink    = new Link();
            Caption defaultCaption = new Caption();

            MyNode    myNode    = null;
            MyLink    myLink    = null;
            MyCaption myCaption = null;

            List<Item> itemList = new List<Item>();

            LoadState loadState          = LoadState.Initial;
            string    version            = addFlow.ProductVersion;
            bool      isVersionLessThan3 = false;

            List<Image> imageList = new List<Image>();

            bool isGradient = true;

            while(xmlTextReader.Read())
            {
                if(xmlTextReader.NodeType == XmlNodeType.Element)
                {
                    switch(xmlTextReader.Name)
                    {
                        case "AddFlow" :

                            loadState = LoadState.AddFlow;

                            int nodeCount = XmlConvert.ToInt32(xmlTextReader.GetAttribute(0));
                            int linkCount = XmlConvert.ToInt32(xmlTextReader.GetAttribute(1));

                            break;

                        case "Version" :
                        {
                            version = xmlTextReader.ReadElementString();

                            string versionString = version.Substring(0, 1);

                            isVersionLessThan3 = (versionString == "1" || versionString == "2");

                            isGradient = !isVersionLessThan3;
                        }
                            break;

                        case "NodeModel"   :
                        case "DefaultNode" :

                            loadState = LoadState.NodeModel;

                            break;

                        case "LinkModel"   :
                        case "DefaultLink" :

                            loadState = LoadState.LinkModel;

                            break;

                        case "CaptionModel" :

                            loadState = LoadState.CaptionModel;

                            break;

                        case "Node" :

                            loadState = LoadState.Node;

                            myNode = new MyNode(defaultNode);

                            if(xmlTextReader.AttributeCount == 5)
                            {
                                int nodeIndex = XmlConvert.ToInt32(xmlTextReader.GetAttribute("Index"));
                            }

                            myNode.Location = new PointF
                            (
                                XmlConvert.ToSingle(xmlTextReader.GetAttribute("Left")),
                                XmlConvert.ToSingle(xmlTextReader.GetAttribute("Top" ))
                            );

                            myNode.Size = new SizeF
                            (
                                XmlConvert.ToSingle(xmlTextReader.GetAttribute("Width" )),
                                XmlConvert.ToSingle(xmlTextReader.GetAttribute("Height"))
                            );

                            itemList.Add(myNode);

                            break;

                        case "Link" :

                            loadState = LoadState.Link;

                            myLink = new MyLink(defaultLink);

                            if(xmlTextReader.AttributeCount == 3)
                            {
                                int linkIndex = XmlConvert.ToInt32(xmlTextReader.GetAttribute("Index"));
                            }

                            myLink.StartNodeIndex = XmlConvert.ToInt32(xmlTextReader.GetAttribute("Org"));
                            myLink.EndNodeIndex = XmlConvert.ToInt32(xmlTextReader.GetAttribute("Dst"));

                            itemList.Add(myLink);

                            break;

                        case "Caption" :

                            loadState = LoadState.Caption;

                            myCaption = new MyCaption(defaultCaption);

                            if(xmlTextReader.AttributeCount == 2)
                            {
                                int captionIndex = XmlConvert.ToInt32(xmlTextReader.GetAttribute("Index"));
                            }

                            myCaption.OwnerIndex = XmlConvert.ToInt32(xmlTextReader.GetAttribute("Owner"));

                            myCaption.Location = new PointF
                            (
                                XmlConvert.ToSingle(xmlTextReader.GetAttribute("Left")),
                                XmlConvert.ToSingle(xmlTextReader.GetAttribute("Top" ))
                            );

                            myCaption.Size = new SizeF
                            (
                                XmlConvert.ToSingle(xmlTextReader.GetAttribute("Width" )),
                                XmlConvert.ToSingle(xmlTextReader.GetAttribute("Height"))
                            );

                            itemList.Add(myCaption);

                            break;

                        case "Pin" :

                            switch(loadState)
                            {
                                case LoadState.NodeModel :
                                {
                                    float x = XmlConvert.ToSingle(xmlTextReader.GetAttribute(0));
                                    float y = XmlConvert.ToSingle(xmlTextReader.GetAttribute(1));

                                    if(defaultNode.Pins == null)
                                    {
                                        defaultNode.Pins = new List<PointF>();
                                    }

                                    defaultNode.Pins.Add(new PointF(x, y));

                                    break;
                                }

                                case LoadState.Node :
                                {
                                    float x = XmlConvert.ToSingle(xmlTextReader.GetAttribute(0));
                                    float y = XmlConvert.ToSingle(xmlTextReader.GetAttribute(1));

                                    if(myNode.Pins == null)
                                    {
                                        myNode.Pins = new List<PointF>();
                                    }

                                    myNode.pinList.Add(new PointF(x, y));

                                    break;
                                }
                            }

                            break;

                        case "Font" :
                        {
                            Font font = ReadFontFromXML(xmlTextReader);

                            switch(loadState)
                            {
                                case LoadState.NodeModel    : defaultNode.Font    = font; break;
                                case LoadState.LinkModel    : defaultLink.Font    = font; break;
                                case LoadState.CaptionModel : defaultCaption.Font = font; break;
                                case LoadState.Node         : myNode.Font         = font; break;
                                case LoadState.Link         : myLink.Font         = font; break;
                                case LoadState.Caption      : myCaption.Font      = font; break;
                            }

                            break;
                        }
                        case "DrawColor" :
                        {
                            Color color = Color.FromArgb(XmlConvert.ToInt32(xmlTextReader.ReadElementString()));

                            switch(loadState)
                            {
                                case LoadState.NodeModel    : defaultNode.DrawColor    = color; break;
                                case LoadState.LinkModel    : defaultLink.DrawColor    = color; break;
                                case LoadState.CaptionModel : defaultCaption.DrawColor = color; break;
                                case LoadState.Node         : myNode.DrawColor         = color; break;
                                case LoadState.Link         : myLink.DrawColor         = color; break;
                                case LoadState.Caption      : myCaption.DrawColor      = color; break;
                            }

                            break;
                        }
                        case "TextColor" :
                        {
                            Color color = Color.FromArgb(XmlConvert.ToInt32(xmlTextReader.ReadElementString()));

                            switch(loadState)
                            {
                                case LoadState.NodeModel    : defaultNode.TextColor    = color; break;
                                case LoadState.LinkModel    : defaultLink.TextColor    = color; break;
                                case LoadState.CaptionModel : defaultCaption.TextColor = color; break;
                                case LoadState.Node         : myNode.TextColor         = color; break;
                                case LoadState.Link         : myLink.TextColor         = color; break;
                                case LoadState.Caption      : myCaption.TextColor      = color; break;
                            }

                            break;
                        }
                        case "FillColor" :
                        {
                            Color color = Color.FromArgb(XmlConvert.ToInt32(xmlTextReader.ReadElementString()));

                            switch(loadState)
                            {
                                case LoadState.NodeModel    : defaultNode.FillColor    = color; break;
                                case LoadState.LinkModel    : defaultLink.FillColor    = color; break;
                                case LoadState.CaptionModel : defaultCaption.FillColor = color; break;
                                case LoadState.Node         : myNode.FillColor         = color; break;
                                case LoadState.Link         : myLink.FillColor         = color; break;
                                case LoadState.Caption      : myCaption.FillColor      = color; break;
                            }

                            break;
                        }
                        case "GradientColor" :
                        {
                            Color color = Color.FromArgb(XmlConvert.ToInt32(xmlTextReader.ReadElementString()));

                            switch(loadState)
                            {
                                case LoadState.NodeModel    : defaultNode.GradientColor    = color; break;
                                case LoadState.CaptionModel : defaultCaption.GradientColor = color; break;
                                case LoadState.Node         : myNode.GradientColor         = color; break;
                                case LoadState.Caption      : myCaption.GradientColor      = color; break;
                            }

                            break;
                        }
                        case "Point" :

                            if(loadState == LoadState.Link)
                            {
                                float x = XmlConvert.ToSingle(xmlTextReader.GetAttribute(0));
                                float y = XmlConvert.ToSingle(xmlTextReader.GetAttribute(1));

                                myLink.PointList.Add(new PointF(x, y));
                            }

                            break;

                        case "DashStyle" :
                        {
                            DashStyle dashStyle = (DashStyle)Enum.Parse(typeof(DashStyle), xmlTextReader.ReadElementString(), false);

                            switch(loadState)
                            {
                                case LoadState.NodeModel    : defaultNode.DashStyle    = dashStyle; break;
                                case LoadState.LinkModel    : defaultLink.DashStyle    = dashStyle; break;
                                case LoadState.CaptionModel : defaultCaption.DashStyle = dashStyle; break;
                                case LoadState.Node         : myNode.DashStyle         = dashStyle; break;
                                case LoadState.Link         : myLink.DashStyle         = dashStyle; break;
                                case LoadState.Caption      : myCaption.DashStyle      = dashStyle; break;
                            }

                            break;
                        }
                        case "Thickness" :
                        case "DrawWidth" :
                        {
                            float value = XmlConvert.ToSingle(xmlTextReader.ReadElementString());

                            switch(loadState)
                            {
                                case LoadState.NodeModel    : defaultNode.Thickness    = value; break;
                                case LoadState.LinkModel    : defaultLink.Thickness    = value; break;
                                case LoadState.CaptionModel : defaultCaption.Thickness = value; break;
                                case LoadState.Node         : myNode.Thickness         = value; break;
                                case LoadState.Link         : myLink.Thickness         = value; break;
                                case LoadState.Caption      : myCaption.Thickness      = value; break;
                            }

                            break;
                        }
                        case "BackMode" :
                        {
                            BackMode backMode = (BackMode)Enum.Parse(typeof(BackMode), xmlTextReader.ReadElementString(), false);

                            switch(loadState)
                            {
                                case LoadState.NodeModel    : defaultNode.BackMode    = backMode; break;
                                case LoadState.LinkModel    : defaultLink.BackMode    = backMode; break;
                                case LoadState.CaptionModel : defaultCaption.BackMode = backMode; break;
                                case LoadState.Node         : myNode.BackMode         = backMode; break;
                                case LoadState.Link         : myLink.BackMode         = backMode; break;
                                case LoadState.Caption      : myCaption.BackMode      = backMode; break;
                            }

                            break;
                        }
                        case "ShapeFamily" :
                        {
                            ShapeFamily shapeFamily = (ShapeFamily)Enum.Parse(typeof(ShapeFamily), xmlTextReader.ReadElementString(), false);

                            switch(loadState)
                            {
                                case LoadState.NodeModel    : defaultNode.ShapeFamily    = shapeFamily; break;
                                case LoadState.CaptionModel : defaultCaption.ShapeFamily = shapeFamily; break;
                                case LoadState.Node         : myNode.ShapeFamily         = shapeFamily; break;
                                case LoadState.Caption      : myCaption.ShapeFamily      = shapeFamily; break;
                            }

                            break;
                        }
                        case "ShadowStyle" :
                        {
                            ShadowStyle shadowStyle = (ShadowStyle)Enum.Parse(typeof(ShadowStyle), xmlTextReader.ReadElementString(), false);

                            switch(loadState)
                            {
                                case LoadState.NodeModel    : defaultNode.ShadowStyle    = shadowStyle; break;
                                case LoadState.LinkModel    : defaultLink.ShadowStyle    = shadowStyle; break;
                                case LoadState.CaptionModel : defaultCaption.ShadowStyle = shadowStyle; break;
                                case LoadState.Node         : myNode.ShadowStyle         = shadowStyle; break;
                                case LoadState.Link         : myLink.ShadowStyle         = shadowStyle; break;
                                case LoadState.Caption      : myCaption.ShadowStyle      = shadowStyle; break;
                            }

                            break;
                        }
                        case "ShadowColor" :
                        {
                            Color color = Color.FromArgb(XmlConvert.ToInt32(xmlTextReader.ReadElementString()));

                            switch(loadState)
                            {
                                case LoadState.NodeModel    : defaultNode.ShadowColor    = color; break;
                                case LoadState.LinkModel    : defaultLink.ShadowColor    = color; break;
                                case LoadState.CaptionModel : defaultCaption.ShadowColor = color; break;
                                case LoadState.Node         : myNode.ShadowColor         = color; break;
                                case LoadState.Link         : myLink.ShadowColor         = color; break;
                                case LoadState.Caption      : myCaption.ShadowColor      = color; break;
                            }

                            break;
                        }
                        case "ShadowSize" :
                        {
                            Size size = ReadShadowSizeFromXML(xmlTextReader);

                            switch(loadState)
                            {
                                case LoadState.NodeModel    : defaultNode.ShadowSize    = size; break;
                                case LoadState.LinkModel    : defaultLink.ShadowSize    = size; break;
                                case LoadState.CaptionModel : defaultCaption.ShadowSize = size; break;
                                case LoadState.Node         : myNode.ShadowSize         = size; break;
                                case LoadState.Link         : myLink.ShadowSize         = size; break;
                                case LoadState.Caption      : myCaption.ShadowSize      = size; break;
                            }

                            break;
                        }
                        case "Alignment"    :
                        case "TextPosition" :
                        {
                            string value = xmlTextReader.ReadElementString();

                            switch(value)
                            {
                                case "LeftJustifyTOP"     : value = "LeftTop";      break;
                                case "LeftJustifyMIDDLE"  : value = "LeftMiddle";   break;
                                case "LeftJustifyBOTTOM"  : value = "LeftBottom";   break;
                                case "RightJustifyTOP"    : value = "RightTop";     break;
                                case "RightJustifyMIDDLE" : value = "RightMiddle";  break;
                                case "RightJustifyBOTTOM" : value = "RightBottom";  break;
                                case "CenterTOP"          : value = "CenterTop";    break;
                                case "CenterMIDDLE"       : value = "CenterMiddle"; break;
                                case "CenterBOTTOM"       : value = "CenterBottom"; break;
                                default                   : break;
                            }

                            TextPosition textPosition = (TextPosition)Enum.Parse(typeof(TextPosition), value, false);

                            switch(loadState)
                            {
                                case LoadState.NodeModel    : defaultNode.TextPosition    = textPosition; break;
                                case LoadState.CaptionModel : defaultCaption.TextPosition = textPosition; break;
                                case LoadState.Node         : myNode.TextPosition         = textPosition; break;
                                case LoadState.Caption      : myCaption.TextPosition      = textPosition; break;
                            }

                            break;
                        }
                        case "ImagePosition" :
                        {
                            string value = xmlTextReader.ReadElementString();

                            if(value != "RelativeToText" && value != "Custom")
                            {
                                ImagePosition imagePosition = (ImagePosition)Enum.Parse(typeof(ImagePosition), value, false);

                                switch(loadState)
                                {
                                    case LoadState.NodeModel    : defaultNode.ImagePosition    = imagePosition; break;
                                    case LoadState.CaptionModel : defaultCaption.ImagePosition = imagePosition; break;
                                    case LoadState.Node         : myNode.ImagePosition         = imagePosition; break;
                                    case LoadState.Caption      : myCaption.ImagePosition      = imagePosition; break;
                                }
                            }

                            break;
                        }
                        case "Dock" :
                        {
                            DockStyle dockStyle = (DockStyle)Enum.Parse(typeof(DockStyle), xmlTextReader.ReadElementString(), false);

                            switch(loadState)
                            {
                                case LoadState.CaptionModel : defaultCaption.Dock = dockStyle; break;
                                case LoadState.Caption      : myCaption.Dock      = dockStyle; break;
                            }

                            break;
                        }
                        case "AnchorPositionOnLink" :
                        {
                            float value = XmlConvert.ToSingle(xmlTextReader.ReadElementString());

                            switch(loadState)
                            {
                                case LoadState.CaptionModel : defaultCaption.AnchorPositionOnLink = value; break;
                                case LoadState.Caption      : myCaption.AnchorPositionOnLink      = value; break;
                            }

                            break;
                        }
                        case "TextMargin" :
                        {
                            Padding margin = ReadTextMarginFromXML(xmlTextReader);

                            switch(loadState)
                            {
                                case LoadState.NodeModel    : defaultNode.TextMargin    = margin; break;
                                case LoadState.CaptionModel : defaultCaption.TextMargin = margin; break;
                                case LoadState.Node         : myNode.TextMargin         = margin; break;
                                case LoadState.Caption      : myCaption.TextMargin      = margin; break;
                            }

                            break;
                        }
                        case "ImageMargin" :
                        {
                            Padding margin = ReadImageMarginFromXML(xmlTextReader);

                            switch(loadState)
                            {
                                case LoadState.NodeModel    : defaultNode.ImageMargin    = margin; break;
                                case LoadState.CaptionModel : defaultCaption.ImageMargin = margin; break;
                                case LoadState.Node         : myNode.ImageMargin         = margin; break;
                                case LoadState.Caption      : myCaption.ImageMargin      = margin; break;
                            }

                            break;
                        }
                        case "GradientMode" :
                        {
                            LinearGradientMode linearGradientMode = (LinearGradientMode)Enum.Parse(typeof(LinearGradientMode), xmlTextReader.ReadElementString(), false);

                            switch(loadState)
                            {
                                case LoadState.NodeModel    : defaultNode.GradientMode    = linearGradientMode; break;
                                case LoadState.CaptionModel : defaultCaption.GradientMode = linearGradientMode; break;
                                case LoadState.Node         : myNode.GradientMode         = linearGradientMode; break;
                                case LoadState.Caption      : myCaption.GradientMode      = linearGradientMode; break;
                            }

                            break;
                        }
                        case "LineStyle" :
                        {
                            LineStyle lineStyle = (LineStyle)Enum.Parse(typeof(LineStyle), xmlTextReader.ReadElementString(), false);

                            if(loadState == LoadState.LinkModel)
                            {
                                defaultLink.LineStyle = lineStyle;
                            }
                            else if(loadState == LoadState.Link)
                            {
                                myLink.LineStyle = lineStyle;
                            }

                            break;
                        }
                        case "RoundedCornerSize" :
                        {
                            float value = XmlConvert.ToSingle(xmlTextReader.ReadElementString());

                            if(loadState == LoadState.Link)
                            {
                                myLink.RoundedCornerSize = value;
                            }
                            else if(loadState == LoadState.LinkModel)
                            {
                                defaultLink.RoundedCornerSize = value;
                            }

                            break;
                        }
                        case "JumpSize" :
                        {
                            float jumpSize = XmlConvert.ToSingle(xmlTextReader.ReadElementString());

                            if(loadState == LoadState.Link)
                            {
                                myLink.JumpSize = jumpSize;
                            }
                            else if(loadState == LoadState.LinkModel)
                            {
                                defaultLink.JumpSize = jumpSize;
                            }

                            break;
                        }
                        case "Jump" :
                        {
                            Jump jump = (Jump)Enum.Parse(typeof(Jump), xmlTextReader.ReadElementString(), false);

                            if(jump == Jump.None)
                            {
                                if(loadState == LoadState.Link)
                                {
                                    myLink.JumpSize = 0;
                                }
                                else if(loadState == LoadState.LinkModel)
                                {
                                    defaultLink.JumpSize = 0;
                                }
                            }

                            break;
                        }
                        case "IsHitTestVisible" :
                        {
                            bool value = (xmlTextReader.ReadElementString() == "True");

                            switch(loadState)
                            {
                                case LoadState.NodeModel    : defaultNode.IsHitTestVisible    = value; break;
                                case LoadState.LinkModel    : defaultLink.IsHitTestVisible    = value; break;
                                case LoadState.CaptionModel : defaultCaption.IsHitTestVisible = value; break;
                                case LoadState.Node         : myNode.IsHitTestVisible         = value; break;
                                case LoadState.Link         : myLink.IsHitTestVisible         = value; break;
                                case LoadState.Caption      : myCaption.IsHitTestVisible      = value; break;
                            }

                            break;
                        }
                        case "IsSelectable" :
                        case "Selectable"   :
                        {
                            bool value = (xmlTextReader.ReadElementString() == "True");

                            switch(loadState)
                            {
                                case LoadState.NodeModel    : defaultNode.IsSelectable    = value; break;
                                case LoadState.LinkModel    : defaultLink.IsSelectable    = value; break;
                                case LoadState.CaptionModel : defaultCaption.IsSelectable = value; break;
                                case LoadState.Node         : myNode.IsSelectable         = value; break;
                                case LoadState.Link         : myLink.IsSelectable         = value; break;
                                case LoadState.Caption      : myCaption.IsSelectable      = value; break;
                            }

                            break;
                        }
                        case "IsOwnerDraw" :
                        case "OwnerDraw"   :
                        {
                            bool value = (xmlTextReader.ReadElementString() == "True");

                            switch(loadState)
                            {
                                case LoadState.NodeModel    : defaultNode.IsOwnerDraw    = value; break;
                                case LoadState.LinkModel    : defaultLink.IsOwnerDraw    = value; break;
                                case LoadState.CaptionModel : defaultCaption.IsOwnerDraw = value; break;
                                case LoadState.Node         : myNode.IsOwnerDraw         = value; break;
                                case LoadState.Link         : myLink.IsOwnerDraw         = value; break;
                                case LoadState.Caption      : myCaption.IsOwnerDraw      = value; break;
                            }

                            break;
                        }
                        case "IsCaptionsHilighted" :
                        {
                            bool value = (xmlTextReader.ReadElementString() == "True");

                            switch(loadState)
                            {
                                case LoadState.NodeModel : defaultNode.IsCaptionsHilighted = value; break;
                                case LoadState.LinkModel : defaultLink.IsCaptionsHilighted = value; break;
                                case LoadState.Node      : myNode.IsCaptionsHilighted      = value; break;
                                case LoadState.Link      : myLink.IsCaptionsHilighted      = value; break;
                            }

                            break;
                        }
                        case "IsXMoveable" :
                        case "XMoveable"   :
                        {
                            bool value = (xmlTextReader.ReadElementString() == "True");

                            switch(loadState)
                            {
                                case LoadState.NodeModel    : defaultNode.IsXMoveable    = value; break;
                                case LoadState.CaptionModel : defaultCaption.IsXMoveable = value; break;
                                case LoadState.Node         : myNode.IsXMoveable         = value; break;
                                case LoadState.Caption      : myCaption.IsXMoveable      = value; break;
                            }

                            break;
                        }
                        case "IsYMoveable" :
                        case "YMoveable"   :
                        {
                            bool value = (xmlTextReader.ReadElementString() == "True");

                            switch(loadState)
                            {
                                case LoadState.NodeModel    : defaultNode.IsYMoveable    = value; break;
                                case LoadState.CaptionModel : defaultCaption.IsYMoveable = value; break;
                                case LoadState.Node         : myNode.IsYMoveable         = value; break;
                                case LoadState.Caption      : myCaption.IsYMoveable      = value; break;
                            }

                            break;
                        }
                        case "IsXSizeable" :
                        case "XSizeable"   :
                        {
                            bool value = (xmlTextReader.ReadElementString() == "True");

                            switch(loadState)
                            {
                                case LoadState.NodeModel    : defaultNode.IsXSizeable    = value; break;
                                case LoadState.CaptionModel : defaultCaption.IsXSizeable = value; break;
                                case LoadState.Node         : myNode.IsXSizeable         = value; break;
                                case LoadState.Caption      : myCaption.IsXSizeable      = value; break;
                            }

                            break;
                        }
                        case "IsYSizeable" :
                        case "YSizeable"   :
                        {
                            bool value = (xmlTextReader.ReadElementString() == "True");

                            switch(loadState)
                            {
                                case LoadState.NodeModel    : defaultNode.IsYSizeable    = value; break;
                                case LoadState.CaptionModel : defaultCaption.IsYSizeable = value; break;
                                case LoadState.Node         : myNode.IsYSizeable         = value; break;
                                case LoadState.Caption      : myCaption.IsYSizeable      = value; break;
                            }

                            break;
                        }
                        case "IsEditable" :
                        case "LabelEdit"  :
                        {
                            bool value = (xmlTextReader.ReadElementString() == "True");

                            switch(loadState)
                            {
                                case LoadState.NodeModel    : defaultNode.IsEditable    = value; break;
                                case LoadState.CaptionModel : defaultCaption.IsEditable = value; break;
                                case LoadState.Node         : myNode.IsEditable         = value; break;
                                case LoadState.Caption      : myCaption.IsEditable      = value; break;
                            }

                            break;
                        }
                        case "IsInLinkable" :
                        case "InLinkable"   :
                        {
                            bool value = (xmlTextReader.ReadElementString() == "True");

                            if(loadState == LoadState.Node)
                            {
                                myNode.IsInLinkable = value;
                            }
                            else if(loadState == LoadState.NodeModel)
                            {
                                defaultNode.IsInLinkable = value;
                            }

                            break;
                        }
                        case "IsOutLinkable" :
                        case "OutLinkable"   :
                        {
                            bool value = (xmlTextReader.ReadElementString() == "True");

                            if(loadState == LoadState.Node)
                            {
                                myNode.IsOutLinkable = value;
                            }
                            else if(loadState == LoadState.NodeModel)
                            {
                                defaultNode.IsOutLinkable = value;
                            }

                            break;
                        }
                        case "IsAdjustDst" :
                        case "AdjustDst"   :
                        {
                            bool value = (xmlTextReader.ReadElementString() == "True");

                            if(loadState == LoadState.Link)
                            {
                                myLink.IsAdjustDst = value;
                            }
                            else if(loadState == LoadState.LinkModel)
                            {
                                defaultLink.IsAdjustDst = value;
                            }

                            break;
                        }
                        case "IsAdjustOrg" :
                        case "AdjustOrg"   :
                        {
                            bool value = (xmlTextReader.ReadElementString() == "True");

                            if(loadState == LoadState.Link)
                            {
                                myLink.IsAdjustOrg = value;
                            }
                            else if(loadState == LoadState.LinkModel)
                            {
                                defaultLink.IsAdjustOrg = value;
                            }

                            break;
                        }
                        case "IsOrientedText" :
                        case "OrientedText"   :
                        {
                            bool value = (xmlTextReader.ReadElementString() == "True");

                            if(loadState == LoadState.Link)
                            {
                                myLink.IsOrientedText = value;
                            }
                            else if(loadState == LoadState.LinkModel)
                            {
                                defaultLink.IsOrientedText = value;
                            }

                            break;
                        }
                        case "IsStretchable" :
                        case "Stretchable"   :
                        {
                            bool value = (xmlTextReader.ReadElementString() == "True");

                            if(loadState == LoadState.Link)
                            {
                                myLink.IsStretchable = value;
                            }
                            else if(loadState == LoadState.LinkModel)
                            {
                                defaultLink.IsStretchable = value;
                            }

                            break;
                        }
                        case "IsArrowDst" :
                        {
                            bool value = (xmlTextReader.ReadElementString() == "True");

                            if(loadState == LoadState.Link)
                            {
                                myLink.IsArrowDst = value;
                            }
                            else if(loadState == LoadState.LinkModel)
                            {
                                defaultLink.IsArrowDst = value;
                            }

                            break;
                        }
                        case "IsArrowOrg" :
                        {
                            bool value = (xmlTextReader.ReadElementString() == "True");

                            if(loadState == LoadState.Link)
                            {
                                myLink.IsArrowOrg = value;
                            }
                            else if(loadState == LoadState.LinkModel)
                            {
                                defaultLink.IsArrowOrg = value;
                            }

                            break;
                        }
                        case "IsArrowMid" :
                        {
                            bool value = (xmlTextReader.ReadElementString() == "True");

                            if(loadState == LoadState.Link)
                            {
                                myLink.IsArrowMid = value;
                            }
                            else if(loadState == LoadState.LinkModel)
                            {
                                defaultLink.IsArrowMid = value;
                            }

                            break;
                        }
                        case "Gradient" :
                        {
                            isGradient = (xmlTextReader.ReadElementString() == "True");

                            break;
                        }
                        case "Image" :

                            if(isVersionLessThan3)
                            {
                                imageList.Add(ReadImageFromXML(xmlTextReader));
                            }
                            else
                            {
                                switch(loadState)
                                {
                                    case LoadState.NodeModel    : defaultNode.Image    = ReadImageFromXML(xmlTextReader); break;
                                    case LoadState.CaptionModel : defaultCaption.Image = ReadImageFromXML(xmlTextReader); break;
                                    case LoadState.Node         : myNode.Image         = ReadImageFromXML(xmlTextReader); break;
                                    case LoadState.Caption      : myCaption.Image      = ReadImageFromXML(xmlTextReader); break;
                                }
                            }

                            break;

                        case "ImageIndex" :
                        {
                            int index =  XmlConvert.ToInt32(xmlTextReader.ReadElementString());

                            if(loadState == LoadState.Node)
                            {
                                myNode.Image = imageList[index];
                            }
                            else if(loadState == LoadState.NodeModel)
                            {
                                defaultNode.Image = imageList[index];
                            }

                            break;
                        }
                        case "Text" :
                        {
                            string value = xmlTextReader.ReadElementString();

                            switch(loadState)
                            {
                                case LoadState.NodeModel    : defaultNode.Text    = value; break;
                                case LoadState.LinkModel    : defaultLink.Text    = value; break;
                                case LoadState.CaptionModel : defaultCaption.Text = value; break;
                                case LoadState.Node         : myNode.Text         = value; break;
                                case LoadState.Link         : myLink.Text         = value; break;
                                case LoadState.Caption      : myCaption.Text      = value; break;
                            }

                            break;
                        }
                        case "Tooltip" :
                        {
                            string value = xmlTextReader.ReadElementString();

                            switch(loadState)
                            {
                                case LoadState.NodeModel    : defaultNode.Tooltip    = value; break;
                                case LoadState.LinkModel    : defaultLink.Tooltip    = value; break;
                                case LoadState.CaptionModel : defaultCaption.Tooltip = value; break;
                                case LoadState.Node         : myNode.Tooltip         = value; break;
                                case LoadState.Link         : myLink.Tooltip         = value; break;
                                case LoadState.Caption      : myCaption.Tooltip      = value; break;
                            }

                            break;
                        }
                        case "Transparent" :
                        {
                            bool isTransparent = (xmlTextReader.ReadElementString() == "True");

                            if(isTransparent)
                            {
                                if(loadState == LoadState.Node)
                                {
                                    myNode.FillColor = Color.Transparent;
                                }
                                else if(loadState == LoadState.NodeModel)
                                {
                                    defaultNode.FillColor = Color.Transparent;
                                }
                            }

                            break;
                        }
                        case "Hidden" :
                        {
                            bool isHidden = (xmlTextReader.ReadElementString() == "True");

                            if(isHidden)
                            {
                                switch(loadState)
                                {
                                    case LoadState.NodeModel :

                                        defaultNode.FillColor = Color.Transparent;
                                        defaultNode.DrawColor = Color.Transparent;

                                        break;

                                    case LoadState.LinkModel :

                                        defaultLink.FillColor = Color.Transparent;
                                        defaultLink.DrawColor = Color.Transparent;

                                        break;

                                    case LoadState.Node :

                                        myNode.FillColor = Color.Transparent;
                                        myNode.DrawColor = Color.Transparent;

                                        break;

                                    case LoadState.Link :

                                        myLink.FillColor = Color.Transparent;
                                        myLink.DrawColor = Color.Transparent;

                                        break;
                                }
                            }

                            break;
                        }
                        case "Shadow" :
                        {
                            ShadowStyle shadowStyle = (ShadowStyle)Enum.Parse(typeof(ShadowStyle), xmlTextReader.GetAttribute(0), false);
                            Color       color       = Color.FromArgb(XmlConvert.ToInt32(xmlTextReader.GetAttribute(1)));
                            Size        size        = new Size(XmlConvert.ToInt32(xmlTextReader.GetAttribute(2)), XmlConvert.ToInt32(xmlTextReader.GetAttribute(3)));

                            switch(loadState)
                            {
                                case LoadState.NodeModel :

                                    defaultNode.ShadowStyle = shadowStyle;
                                    defaultNode.ShadowColor = color;
                                    defaultNode.ShadowSize  = size;

                                    break;

                                case LoadState.LinkModel :

                                    defaultLink.ShadowStyle = shadowStyle;
                                    defaultLink.ShadowColor = color;
                                    defaultLink.ShadowSize  = size;

                                    break;

                                case LoadState.Node :

                                    myNode.ShadowStyle = shadowStyle;
                                    myNode.ShadowColor = color;
                                    myNode.ShadowSize  = size;

                                    break;

                                case LoadState.Link :

                                    myLink.ShadowStyle = shadowStyle;
                                    myLink.ShadowColor = color;
                                    myLink.ShadowSize  = size;

                                    break;
                            }

                            break;
                        }
                        case "Shape" :
                        {
                            NodeShapeStyle   nodeShapeStyle   = (NodeShapeStyle)Enum.Parse(typeof(NodeShapeStyle), xmlTextReader.GetAttribute(0), false);
                            ShapeOrientation shapeOrientation = (ShapeOrientation)Enum.Parse(typeof(ShapeOrientation), xmlTextReader.GetAttribute(1), false);

                            if(loadState == LoadState.NodeModel)
                            {
                                defaultNode.ShapeFamily  = ShapeFamily.Rectangle;
                                defaultNode.GraphicsPath = PredefinedGeometry.GetNodePath(nodeShapeStyle, shapeOrientation);
                            }
                            else if(loadState == LoadState.Node)
                            {
                                myNode.ShapeFamily  = ShapeFamily.Rectangle;
                                myNode.GraphicsPath = PredefinedGeometry.GetNodePath(nodeShapeStyle, shapeOrientation);
                            }

                            break;
                        }
                        case "ArrowDst" :

                            if(loadState == LoadState.LinkModel)
                            {
                                defaultLink.ArrowDst   = GetArrow(xmlTextReader);
                                defaultLink.IsArrowDst = true;
                            }
                            else if(loadState == LoadState.Link)
                            {
                                myLink.ArrowDst   = GetArrow(xmlTextReader);
                                myLink.IsArrowDst = true;
                            }

                            break;

                        case "ArrowOrg" :

                            if(loadState == LoadState.LinkModel)
                            {
                                defaultLink.ArrowOrg   = GetArrow(xmlTextReader);
                                defaultLink.IsArrowOrg = true;
                            }
                            else if(loadState == LoadState.Link)
                            {
                                myLink.ArrowOrg   = GetArrow(xmlTextReader);
                                myLink.IsArrowOrg = true;
                            }

                            break;

                        case "ArrowMid" :

                            if(loadState == LoadState.LinkModel)
                            {
                                defaultLink.ArrowMid   = GetArrow(xmlTextReader);
                                defaultLink.IsArrowMid = true;
                            }
                            else if(loadState == LoadState.Link)
                            {
                                myLink.ArrowMid   = GetArrow(xmlTextReader);
                                myLink.IsArrowMid = true;
                            }

                            break;

                        case "Line" :
                        {
                            LineStyle lineStyle = (LineStyle)Enum.Parse(typeof(LineStyle), xmlTextReader.GetAttribute(0), false);

                            bool isRoundedCorner = (xmlTextReader.GetAttribute(2) == "True");

                            if(loadState == LoadState.LinkModel)
                            {
                                defaultLink.LineStyle = lineStyle;

                                if(isRoundedCorner)
                                {
                                    defaultLink.RoundedCornerSize = SystemInformation.IconSize.Width / 3;
                                }
                                else
                                {
                                    defaultLink.RoundedCornerSize = 0;
                                }
                            }
                            else if(loadState == LoadState.Link)
                            {
                                myLink.LineStyle = lineStyle;

                                if(isRoundedCorner)
                                {
                                    myLink.RoundedCornerSize = SystemInformation.IconSize.Width / 3;
                                }
                                else
                                {
                                    myLink.RoundedCornerSize = 0;
                                }
                            }

                            break;
                        }
                    }
                }

                if(!isGradient)
                {
                    if(loadState == LoadState.Node)
                    {
                        myNode.GradientColor = myNode.FillColor;
                    }
                    else if(loadState == LoadState.NodeModel)
                    {
                        defaultNode.GradientColor = defaultNode.FillColor;
                    }
                }
            }

            addFlow.BeginUpdate();

            foreach(object item in itemList)
            {
                if(item is MyNode)
                {
                    Node node = item as Node;

                    myNode = item as MyNode;

                    if(myNode.pinList.Count > 0)
                    {
                        node.Pins = myNode.pinList;
                    }

                    addFlow.AddNode(node);
                }
            }

            foreach(object item in itemList)
            {
                if(item is MyLink)
                {
                    myLink = item as MyLink;

                    Node startNode = ((Node)itemList[myLink.StartNodeIndex]);
                    Node endNode   = ((Node)itemList[myLink.EndNodeIndex]);

                    myLink.Org = startNode;
                    myLink.Dst = endNode;

                    addFlow.AddLink((Link)myLink);
                                        
                    if(myLink.PointList.Count > 0)
                    {
                        int addCount = myLink.PointList.Count;

                        if(myLink.IsAdjustOrg)
                        {
                            addCount--;
                        }

                        if(myLink.IsAdjustDst)
                        {
                            addCount--;
                        }

                        if(myLink.LineStyle == LineStyle.Polyline || myLink.LineStyle == LineStyle.Spline)
                        {
                            if(myLink.Org == myLink.Dst)
                            {
                                addCount -= 2;
                            }

                            for(int k = 0; k < addCount; k++)
                            {
                                myLink.AddPoint(new PointF(0, 0));
                            }
                        }
            
                        int startIndex = myLink.IsAdjustOrg? 0 : 1;
                        int endIndex   = myLink.IsAdjustDst ? myLink.CountPoints() : myLink.CountPoints() - 1;
                        int i          = 0;

                        for(int k = startIndex; k < endIndex; k++)
                        {
                            myLink.SetPoint((PointF)myLink.PointList[i++], k);
                        }
                    }
                }
            }

            foreach(object item in itemList)
            {
                if(item is MyCaption)
                {
                    myCaption = item as MyCaption;

                    if(myCaption.OwnerIndex >= 0)
                    {
                        myCaption.Owner = itemList[myCaption.OwnerIndex];
                    }

                    addFlow.AddCaption((Caption)myCaption);
                }
            }
            
            for(int i = 0; i < itemList.Count; i++)
            {
                object item = itemList[i];

                if(item is MyNode)
                {
                    Node node = item as Node;

                    node.ZOrder = i;
                }
                else if(item is MyLink)
                {
                    Link link = item as Link;

                    link.ZOrder = i;
                }
                else if(item is MyCaption)
                {
                    Caption caption = item as Caption;

                    caption.ZOrder = i;
                }
            }

            if(selected)
            {
                foreach(Item item in itemList)
                {
                    item.IsSelected = true;
                }
            }

            addFlow.EndUpdate();

            if(acceptDefaultValues)
            {
                addFlow.NodeModel    = defaultNode;
                addFlow.LinkModel    = defaultLink;
                addFlow.CaptionModel = defaultCaption;
            }
        }

        #endregion
    }
}